import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import {
	AuthenticationDetails,
	CognitoUser,
	CognitoUserAttribute,
	CognitoUserPool,
  CognitoUserSession,
} from 'amazon-cognito-identity-js';
import RegisterDto from './dto/register.dto';

@Injectable()
export class AuthenticationService {
  private readonly userPool: CognitoUserPool;
	constructor(
		configService: ConfigService
	) {
    this.userPool = new CognitoUserPool({
      UserPoolId: configService.get('COGNITO_USER_POOL_ID'),
      ClientId: configService.get('COGNITO_CLIENT_ID'),
    });
  }

  getCognitoUser(Username: string) {
    return new CognitoUser({
      Username,
      Pool: this.userPool
    });
  }

  login(name: string, password: string) {
    const newUser = this.getCognitoUser(name);
    const authenticationDetails = new AuthenticationDetails({
      Username: name,
      Password: password
    });

    return new Promise<CognitoUserSession>((resolve, reject) => {
      newUser.authenticateUser(authenticationDetails, {
        onSuccess: result => resolve(result),
        onFailure: err => reject(err)
      });
    });
  }

  private createAttributes(registrationData: RegisterDto) {
    const { name } = registrationData;

    return [
      new CognitoUserAttribute({ Name: 'name', Value: name })
    ];
  }

  register(registrationData: RegisterDto) {
    const { email, password } = registrationData;

    return new Promise<CognitoUser>((resolve, reject) => {
      this.userPool.signUp(
        email,
        password,
        this.createAttributes(registrationData),
        null,
        (error, result) => {
          if (!result) {
            reject(error);
          } else {
            resolve(result.user);
          }
        }
      );
    });
  }

  confirm(email: string, code: string) {
    const userToConfirm = this.getCognitoUser(email);

    return new Promise((resolve, reject) => {
      userToConfirm.confirmRegistration(code, true, (err, result) => {
        if (err) {
          reject(err);
        }
        resolve(result);
      });
    });
  }
}
